import { Component, Input, OnInit } from '@angular/core';
import { UntypedFormArray, UntypedFormGroup } from '@angular/forms';
import { MatDialog } from '@angular/material/dialog';
import { ReferenceType } from '@app/core/model/reference-type/reference-type';
import { ReferenceService } from '@app/core/services/reference/reference.service';
import { EnumUtils } from '@app/core/services/utilities/enum-utils.service';
import { MultipleAutoCompleteConfiguration } from '@app/library/auto-complete/multiple/multiple-auto-complete-configuration';
import { SingleAutoCompleteConfiguration } from '@app/library/auto-complete/single/single-auto-complete-configuration';
import { BaseComponent } from '@common/base/base.component';
import { takeUntil } from 'rxjs/operators';
import { ReferenceDialogEditorComponent } from './editor/reference-dialog-editor.component';
import { Reference, ReferencePersist } from '@app/core/model/reference/reference';
import { ReferenceSearchLookup } from '@app/core/query/reference-search.lookup';
import { Guid } from '@common/types/guid';
import { nameof } from 'ts-simple-nameof';

@Component({
	selector: 'app-reference-field-component',
	templateUrl: 'reference-field.component.html',
	styleUrls: ['./reference-field.component.scss']
})
export class ReferenceFieldComponent extends BaseComponent implements OnInit {

	@Input() referenceType: ReferenceType = null;
	@Input() form: UntypedFormGroup = null;
	@Input() label: string = null;
	@Input() required: boolean = false;
	@Input() multiple: boolean = true;
	@Input() hint: string;
	@Input() placeholder: string;
	@Input() dependencies: UntypedFormGroup = null;

	referenceToUse: Reference[]= [];
	referenceToUseInitialized = false;

	multipleAutoCompleteSearchConfiguration: MultipleAutoCompleteConfiguration;
	singleAutoCompleteSearchConfiguration: SingleAutoCompleteConfiguration;

	constructor(
		private referenceService: ReferenceService,
		public enumUtils: EnumUtils,
		private dialog: MatDialog,
	) { super(); }

	ngOnInit() {
		const referenceDepedencyIds = this.referenceType.definition.sources.filter(x => x.referenceTypeDependencies).flatMap(x => x.referenceTypeDependencies).filter(x => x).map(x => x.id);
		if (referenceDepedencyIds.length > 0) {
			this.referenceToUseInitialized = false;
			this.dependencies.valueChanges.pipe(takeUntil(this._destroyed)).subscribe(changes => {
				this.resolveReferenceDepedency(referenceDepedencyIds);
			});
			this.resolveReferenceDepedency(referenceDepedencyIds);
		} else {
			if (this.multiple) {
				this.multipleAutoCompleteSearchConfiguration = this.referenceService.getMultipleAutoCompleteSearchConfiguration(this.referenceType.id, null);
			} else {
				this.singleAutoCompleteSearchConfiguration = this.referenceService.getSingleAutocompleteSearchConfiguration(this.referenceType.id, null);
			}
		}
	}

	resolveReferenceDepedency(referenceDepedencyIds: Guid[]) {
		const referenceToUse : Reference[]= [];
		Object.keys(this.dependencies.controls).forEach(controlName => {
			// (this.dependencies.get(controlName).get('references').value as Reference[]).filter(x=> sourcesWithDependencies.some(y => y.referenceTypeDependencies) x.type.id == this.referenceType.id &&)
			const foudReferences: any[] = this.dependencies.get(controlName).get('references')?.value;
			if (foudReferences != null) {
				for (let i = 0; i < foudReferences.length; i++) {
					const foudReference = foudReferences[i];
					if (foudReference?.typeId && referenceDepedencyIds.includes(foudReference.typeId)) {
						const typed = foudReference as ReferencePersist;

						referenceToUse.push({
							id: typed.id,
							hash: typed.hash,
							description: typed.description,
							reference: typed.reference,
							abbreviation: typed.abbreviation,
							source: typed.source,
							sourceType: typed.sourceType,
							label: typed.label,
							definition: typed.definition,
							type: {
								id: typed.typeId,
							}
						})
					} else if (foudReference?.type?.id && referenceDepedencyIds.includes(foudReference.type.id)) {
						const typed = foudReference as Reference;
						if (typed != null) referenceToUse.push(typed);
					}
				}
			}
		});
		if (this.referenceToUseInitialized && (!referenceToUse.map(x => x.reference).every(x => this.referenceToUse.map(y => y.reference).includes(x)) ||
			!this.referenceToUse.map(x => x.reference).every(x => referenceToUse.map(y => y.reference).includes(x)))) {
				this.form.setValue(null, {onlySelf: true, emitEvent: false});
			}

		this.referenceToUse = referenceToUse;
		this.referenceToUseInitialized = true;

		if (this.multiple) {
			this.multipleAutoCompleteSearchConfiguration = this.referenceService.getMultipleAutoCompleteSearchConfiguration(this.referenceType.id, this.referenceToUse);
		} else {
			this.singleAutoCompleteSearchConfiguration = this.referenceService.getSingleAutocompleteSearchConfiguration(this.referenceType.id, this.referenceToUse);
		}
	}

	addReference() {

		this.referenceService.searchWithDefinition(this.buildAutocompleteSearchLookup(this.referenceType.id))
		.pipe(takeUntil(this._destroyed))
		.subscribe(
			references => {
				const dialogRef = this.dialog.open(ReferenceDialogEditorComponent, {
					width: '500px',
					restoreFocus: false,
					data: {
						referenceTypeId: this.referenceType.id,
						label: this.label ?? this.referenceType.name,
						existingReferences: references.map(x => x.reference)
					},
				});
				dialogRef.afterClosed()
					.pipe(takeUntil(this._destroyed))
					.subscribe(newResult => {
						if (!newResult) { return; }
						let results = this.form.value as ReferencePersist[];
						if (results == undefined) results = [];
						results.push(newResult);
						this.form.setValue(results);
					});
			}
		);
	}

	private buildAutocompleteSearchLookup(typeId: Guid): ReferenceSearchLookup {
		const lookup: ReferenceSearchLookup = new ReferenceSearchLookup();
		lookup.page = { size: 100, offset: 0 };
		lookup.project = {
			fields: [
				nameof<Reference>(x => x.reference),
			]
		};
		lookup.typeId = typeId;
		lookup.order = { items: [nameof<Reference>(x => x.label)] };

		return lookup;
	}
}
